/*
 *	dc93.h -- Header for dc93-Mach1 Scsi-Device-Driver
 *
 *	Copyright (C) 1994 by kdw
 *	low level scsi driver header for DC93 MACH1
 *	Removable Cartidges supportet
 *	(Driver-Version 6.2.0, Linux-Version 1.2)
 *
 *	<kdw@pri-n.cv.com>
 */

#ifndef _DC93_H
	#define DC93_H
/*
	$Header
*/

#ifndef ASM

/* Prototypes for Mid-Level-Interface follows */

int dc93_detect(Scsi_Host_Template *);
int dc93_queuecommand(Scsi_Cmnd *, void (*done)(Scsi_Cmnd *));
int dc93_abort(Scsi_Cmnd *);
const char *dc93_info(void);
int dc93_reset(Scsi_Cmnd *);
int dc93_biosparam(Disk *, int, int *);

#ifndef NULL
	#define NULL 0
#endif

/* This is the Device-Structure, and the Interface-Routines */

#define DC93  { NULL,NULL,			\
		"dc93", dc93_detect,		\
		NULL,				\
		dc93_info,			\
		NULL,	  			\
		dc93_queuecommand, 		\
		dc93_abort,			\
		dc93_reset, 			\
		NULL, 				\
		dc93_biosparam,			\
		2, 7, 0, 1, 0, 1, 0}
/*              |  |  |  |  |  |  |-  Disable Clustering
 *              |  |  |  |  |  |----  unchecked_isa_dma (Flag 0=unchecked)
 *              |  |  |  |  |-------  present (Flag 0=not present)
 *              |  |  |  |----------  cmd_per_lun (depends on can_queue)
 *              |  |  |-------------  sg_tablesize (scatterd gather)
 *              |  |----------------  this_id (see also DC93_HOSTID)
 *              |-------------------  can_queue (Flag 0=no queueing)
 */

#endif

/*
	This is the DC93-Host-ID
*/

#define DC93_HOSTID	7

/*
	Flag to be a Medium-Changer
*/

#define DC93_MC		1	/* 0 = No Changer */

/*
	Max-Queue-Entries
*/

#define DC93_MAX_Q	30

/*
	Debug-Flags (0 = off; 1 = on)
	may be altered by testing people to get messages
*/

#ifndef DC93_MONITOR
/* certanly i could use #define instead of variables, but for Compatibility
   with my scsi-monitor i prefered variables
*/
static int kdw_info	= 0;	/* Procedure-entry & other important Infos */

static int kdw_i_info	= 0;	/* Interrupt-Call */

static int kdw_c_info	= 0;	/* Command & Bus-Information */

static int kdw_t_info   = 0;	/* Queue-Trace */

static int kdw_io_info	= 0;	/* Low-IO-Information */

static int kdw_dma_info	= 0;	/* DMA-Transfer-Information */

/*---------------------------------------------------------------------*/
/* please don't touch these, because they are nessesary for Monitor-Aid */

#else

int kdw_info;
int kdw_i_info;
int kdw_c_info;
int kdw_t_info;
int kdw_io_info;
int kdw_dma_info;

#endif

/*
	This is the Disc-Geometry is true for my Target 0 (SYQUEST 88MB)
*/

#define T0_HEADS	64	/* Heads for this Unit */
#define T0_SECTORS	32	/* Sectors per Track for this Unit */
#define T0_CYLINDERS	84	/* Cylinders for this Unit */

/*
	Registers
*/

#define OWN_ID			0x00	/* CDB_Size as well */
#define CDB_SIZE		0x00
#define	CONTROL			0x01
#define TIMEOUT			0x02
#define	TOTAL_SECTOR		0x03	/* CDB_1 as well */
#define CDB_1			0x03
#define TOTAL_HEADS		0x04	/* CDB_2 as well */
#define CDB_2			0x04
#define TOTAL_CYLINDER_MSB	0x05	/* CDB_3 as well */
#define CDB_3			0x05
#define TOTAL_CYLINDER_LSB	0x06	/* CDB_4 as well */
#define CDB_4			0x06
#define LOGICAL_ADDR_MSB	0x07	/* CDB_5 as well */
#define CDB_5			0x07
#define LOGICAL_ADDR_2ND	0x08	/* CDB_6 as well */
#define CDB_6			0x08
#define LOGICAL_ADDR_3RD	0x09	/* CDB_7 as well */
#define CDB_7			0x09
#define LOGICAL_ADDR_LSB	0x0A	/* CDB_8 as well */
#define CDB_8			0x0A
#define SECTOR_NUMBER		0x0B	/* CDB_9 as well */
#define CDB_9			0x0B
#define HEAD_NUMBER		0x0C	/* CDB_A as well */
#define CDB_A			0x0C
#define CYLINDER_NUMBER_MSB	0x0D	/* CDB_B as well */
#define CDB_B			0x0D
#define CYLINDER_NUMBER_LSB	0x0E	/* CDB_C as well */
#define CDB_C			0x0E
#define TLUN			0x0F
#define COMMAND_PHASE		0x10
#define SYNC_TRANSFER		0x11
#define TRANSFER_COUNT_MSB	0x12
#define TRANSFER_COUNT_2ND	0x13
#define TRANSFER_COUNT_LSB	0x14
#define DEST_ID			0x15
#define SOURCE_ID		0x16
#define SCSI_STATUS		0x17	/* 08-04 Status, 03-00 Code */
#define COMMAND			0x18	/* SBP = 0x80 (Single-Byte) */
#define DATA			0x19
#define AUX_STATUS		0x1F

/*
	COMMAND defines (Initiator-Commands only)
*/

#define CMD_RESET		0x00	/* Soft-Reset */
#define CMD_ABORT		0x01	/* Command-Abort */
#define CMD_AATN		0x02	/* assert ATN */
#define CMD_NACK		0x03	/* negate ACK-Signal */
#define CMD_DISC		0x04	/* force Disconnect */
#define CMD_SELATN		0x06	/* Select with ATN assertet */
#define CMD_SELWATN		0x07	/* Select without ATN */
#define CMD_SEL_TRAN_ATN	0x08	/* Select and Transfer, ATN assertet */
#define CMD_SEL_TRAN_WATN	0x09	/* Select and Transfer without ATN */
#define CMD_SETIDI		0x0F	/* Set IDI */
#define CMD_TRINFO		0x20	/* Transfer-Info */

/*
	CONTROL defines
*/

#define CNTRL_HSP 		0x01
#define CNTRL_HA  		0x02
#define CNTRL_IDI 		0x04
#define CNTRL_EDI   		0x08
#define CNTRL_HHP 		0x10
#define CNTRL_DM0 		0x20
#define CNTRL_DM1 		0x40
#define CNTRL_DM2 		0x80

/*
	AUX_STATUS
*/

#define STAT_BREAK	0x00	/* Bus not ready to send or recieve */
#define STAT_DBR	0x01	/* Data-Buffer Ready */
#define STAT_PE		0x02	/* Parity Error */
#define STAT_CIP	0x10	/* Command in Progress */
#define STAT_BSY	0x20	/* Command is executing */
#define STAT_LCI	0x40	/* Last Command ignored */
#define STAT_INT	0x80	/* Interrupt Pending */

/*
	SCSI_STATUS
*/

#define ST_RESET_WAD	0x00
#define ST_RESET_AD	0x01	/* Reset-Status with Advanced Feature */

/*
	Success-Interrupts
*/

#define ST_SUCC		0x10
#define ST_SUCC_CONN	0x11
#define ST_SUCC_SEL_TR	0x16
#define ST_SUCC_INFO	0x18

/*
	Abort-Interrupts
*/

#define ST_ABORT_MSGIN	0x20
#define ST_ABORT_SPTR	0x21
#define ST_ABORT_SEL	0x22
#define ST_ABORT_DSEL	0x25
#define ST_ABORT_NMAT	0x27
#define ST_ABORT_INFO	0x28

/*
	Termination-Interrupts
*/

#define ST_TERM_INVAL	0x40
#define ST_TERM_UNEXP	0x41
#define ST_TERM_STMOUT	0x42
#define ST_TERM_PE	0x43
#define ST_TERM_PEATN	0x44
#define ST_TERM_NMAT	0x46
#define ST_TERM_INCORR	0x47
#define ST_TERM_INFO	0x48

/*
	Service-Request-Interrupts
*/

#define ST_SERV_RESEL	0x80
#define ST_SERV_ADRESEL	0x81
#define ST_SERV_DISC	0x85
#define ST_SERV_INFO	0x88

#define ST_UNDEFINED	0xFF		/* Only during Transfer */

/*
	Info-Masks
*/

#define DATA_OUT_PHASE		0
#define	DATA_IN_PHASE		1
#define CMD_PHASE		2
#define STATUS_PHASE		3
#define UNSPEC_INFO_OUT_PHASE	4
#define UNSPEC_INFO_IN_PHASE	5
#define MESSAGE_OUT_PHASE	6
#define MESSAGE_IN_PHASE	7
#define PHASE_MASK		7

/*
	Source-ID-Bits
*/

#define ER			0x80	/* Enable Reselection */
#define ES			0x40	/* Enable Selection */
#define DSP			0x20	/* Disable Select-Parity */
#define SIV			0x08	/* Source ID valid */
#define SI2			0x04	/* Source-ID-2 */
#define SI1			0x02	/* Source-ID-1 */
#define SI0			0x01	/* Source-ID-0 */

/*
	Desitantion-ID-Bits
*/

#define SCC			0x80	/* Select Command Chain */
#define DPD			0x40	/* Data-Phase-Direction */
#define DI2			0x04	/* Destination-ID_2 */
#define DI1			0x02	/* Destination-ID_1 */
#define DI0			0x01	/* Destination-ID_0 */

/* for better Understanding which Direction we have, we use those Definitions */

#define TO_TARGET		0x00
#define FROM_TARGET		0x40

/* -------------------------------------------------------------------------- */

/*
	TLUN-Status-Bits
*/

#define TLV			0x80	/* Identify-Message received */
#define DOK			0x40	/* Initiator enable disconnect */
#define	TL2			0x04	/* Target-LUN_2 */
#define TL1			0x02	/* Target-LUN_1 */
#define TL0			0x01	/* Target-LUN_0 */

/*
	OWN-ID Bits
*/

#define FS1			0x80	/* Input-Clock-Frequency High-Bit */
#define FS0			0x40	/* Input-Clock-Frequency Low-Bit */
#define EHP			0x10	/* Enable Host-Parity (configurable) */
#define EAF			0x08	/* Enable Advanced Feature */
#define ID2			0x04	/* This is used during Arbitration */
#define ID1			0x01	/* same, Bit 2 */
#define ID0			0x00	/* same, Bit 1 */

/*
	Command-Phases
*/

#define NOT_SELECTET		0x00
#define INITIATOR_SEL_ONLY	0x10
#define SEND_ID_MSG_TO_TARGET	0x20
#define START_CMD_PHASE_NOTRANS	0x30
#define TRANS_BYTE_MASK		0x3F	/* Max 15 Bytes may be transfered */
#define SAVE_DP_MSG		0x41	/* Data-Pointer has to be saved */
#define DISCONNECT_MSG		0x42	/* BUS is not yet free */
#define BUS_FREE_MSG		0x43	/* Target has disconnectet */
#define MATCH_CONNECT		0x44	/* Target found, connectet */
#define RCV_ID_MSG		0x45	/* Identifyer received */
#define DATA_IO_SUCCESS		0x46	/* all Data-Bytes has been transfered */
#define START_RCV_PHASE		0x47	/* Target is in RCV-Phase */
#define RCV_TARGET_STATUS	0x50	/* Target Status is stored in TLUN */
#define CMD_COMPLETE		0x60	/* Command completet Successfully */
#define LINKED_COMMAND_COMPL	0x61	/* Linked Command completet */

/*
	REQUESTS
*/

#define REQ_MASK 7
#define REQ_DATAOUT 0
#define REQ_DATAIN 1
#define REQ_CMDOUT 2
#define REQ_STATIN 3
#define REQ_MSGOUT 6
#define REQ_MSGIN 7

#endif
